@@ -267,6 +267,8 @@ pub struct ContainerHostConfig {
     cgroup_parent: Option<String>,
     volume_driver: Option<String>,
     shm_size: Option<u64>,
+    userns_mode: Option<String>,
+    device_cgroup_rules: Option<Vec<String>>,
 }
 
 impl ContainerHostConfig {
@@ -276,6 +278,16 @@ impl ContainerHostConfig {
         }
     }
 
+    pub fn device_cgroup_rules(&mut self, device_cgroup_rules: Vec<String>) -> &mut Self {
+        self.device_cgroup_rules = Some(device_cgroup_rules);
+        self
+    }
+
+    pub fn userns_mode(&mut self, userns_mode: String) -> &mut Self {
+        self.userns_mode = Some(userns_mode);
+        self
+    }
+
     pub fn binds(&mut self, binds: Vec<String>) -> &mut Self {
         self.binds = Some(binds);
         self
